import fs from "fs/promises";
import { MongoLogManager } from "mongodb-log-writer";
import config from "./config.js";
import redact from "mongodb-redact";
class LoggerBase {
    info(id, context, message) {
        this.log("info", id, context, message);
    }
    error(id, context, message) {
        this.log("error", id, context, message);
    }
    debug(id, context, message) {
        this.log("debug", id, context, message);
    }
    notice(id, context, message) {
        this.log("notice", id, context, message);
    }
    warning(id, context, message) {
        this.log("warning", id, context, message);
    }
    critical(id, context, message) {
        this.log("critical", id, context, message);
    }
    alert(id, context, message) {
        this.log("alert", id, context, message);
    }
    emergency(id, context, message) {
        this.log("emergency", id, context, message);
    }
}
class ConsoleLogger extends LoggerBase {
    log(level, id, context, message) {
        message = redact(message);
        console.error(`[${level.toUpperCase()}] ${id.__value} - ${context}: ${message}`);
    }
}
class Logger extends LoggerBase {
    constructor(logWriter, server) {
        super();
        this.logWriter = logWriter;
        this.server = server;
    }
    log(level, id, context, message) {
        message = redact(message);
        const mongoDBLevel = this.mapToMongoDBLogLevel(level);
        this.logWriter[mongoDBLevel]("MONGODB-MCP", id, context, message);
        void this.server.server.sendLoggingMessage({
            level,
            data: `[${context}]: ${message}`,
        });
    }
    mapToMongoDBLogLevel(level) {
        switch (level) {
            case "info":
                return "info";
            case "warning":
                return "warn";
            case "error":
                return "error";
            case "notice":
            case "debug":
                return "debug";
            case "critical":
            case "alert":
            case "emergency":
                return "fatal";
            default:
                return "info";
        }
    }
}
class ProxyingLogger extends LoggerBase {
    constructor() {
        super(...arguments);
        this.internalLogger = new ConsoleLogger();
    }
    log(level, id, context, message) {
        this.internalLogger.log(level, id, context, message);
    }
}
const logger = new ProxyingLogger();
export default logger;
export async function initializeLogger(server) {
    await fs.mkdir(config.logPath, { recursive: true });
    const manager = new MongoLogManager({
        directory: config.logPath,
        retentionDays: 30,
        onwarn: console.warn,
        onerror: console.error,
        gzip: false,
        retentionGB: 1,
    });
    await manager.cleanupOldLogFiles();
    const logWriter = await manager.createLogWriter();
    logger["internalLogger"] = new Logger(logWriter, server);
}
//# sourceMappingURL=logger.js.map