import { z } from "zod";
import { MongoDBToolBase } from "../mongodbTool.js";
export class UpdateOneTool extends MongoDBToolBase {
    constructor() {
        super(...arguments);
        this.name = "update-one";
        this.description = "Updates a single document within the collection based on the filter";
        this.argsShape = {
            collection: z.string().describe("Collection name"),
            database: z.string().describe("Database name"),
            filter: z
                .object({})
                .passthrough()
                .optional()
                .describe("The selection criteria for the update, matching the syntax of the filter argument of db.collection.updateOne()"),
            update: z
                .object({})
                .passthrough()
                .optional()
                .describe("An update document describing the modifications to apply using update operator expressions"),
            upsert: z
                .boolean()
                .optional()
                .describe("Controls whether to insert a new document if no documents match the filter"),
        };
        this.operationType = "update";
    }
    async execute({ database, collection, filter, update, upsert, }) {
        const provider = await this.ensureConnected();
        const result = await provider.updateOne(database, collection, filter, update, {
            upsert,
        });
        let message = "";
        if (result.matchedCount === 0) {
            message = `No documents matched the filter.`;
        }
        else {
            message = `Matched ${result.matchedCount} document(s).`;
            if (result.modifiedCount > 0) {
                message += ` Modified ${result.modifiedCount} document(s).`;
            }
            if (result.upsertedCount > 0) {
                message += ` Upserted ${result.upsertedCount} document(s) (with id: ${result.upsertedId?.toString()}).`;
            }
        }
        return {
            content: [
                {
                    text: message,
                    type: "text",
                },
            ],
        };
    }
}
//# sourceMappingURL=updateOne.js.map